package cz.autocont.iszr.demo;

import java.util.ArrayList;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.Properties;
import java.util.UUID;


import cz.autocont.iszr.demo.log.SOAPFileLoggingHandler;
import cz.autocont.iszr.demo.log.SOAPLoggingHandler;
import cz.isvs.iszr.schemas.iszrabstract.v1.ZadostInfoIszrType;

import jakarta.xml.ws.Binding;
import jakarta.xml.ws.BindingProvider;
import jakarta.xml.ws.handler.Handler;

/**
 * Demonstrace volani webove sluzby ISZR.<br/>
 * Tato trida je spolecnym predkem pro demostrace, ktere volaji webovou sluzbu ISZR.
 * Spolecny predek pro dema, ktera volaji jednotlive sluzby ISZR.
 * <div style="font:italic">Projekt ISZR - (c) AutoCont CZ a.s. All rights reserved</div>
 * 
 * @param <REQ> request, ktery se odesila pri volani webove sluzby
 * @param <RES> response, ktery vrati webova sluzba
 * @author Vladimir Pach
 * @since 0.0.1
 */
public abstract class BaseDemoCall<REQ, RES> implements Demo
{
    /**
     * Vrati konfiguraci dema.
     * 
     * @return Hodnota {@link Execute#config}.
     */
    public Properties getConfig()
    {
        return Execute.config;
    }

    /**
     * Spusti demo webove sluzby volanim metody {@link #demoCallService()}.
     * 
     * @throws DemoException - pokud v demu nastane chyba.
     */
    @Override
    public void execute() throws DemoException
    {
        demoCallService();
    }

    /**
     * Demo volani webove sluzby.
     * 
     * @throws DemoException error on call service
     * @see #createRequestData();
     */
    public void demoCallService() throws DemoException
    {
        // prepare request data
        REQ requestData = createRequestData();

        // call service
        RES response = callService(requestData);

        // process response
        processResponseData(response);
    }

    /**
     * Pripravi vstupni data pro volani webove sluzby
     * 
     * @return vstupni data volani
     */
    protected abstract REQ createRequestData() throws DemoException;

    /**
     * Provede volani webove sluzby
     * 
     * @param requestData vstupni data volani. Tato data pripravila metoda {@link #createRequestData()}.
     * @return Vysledek, ktery vratila vebova sluzba.
     */
    protected abstract RES callService(REQ requestData) throws DemoException;

    /**
     * Zpracovani navratove hodnoty z webove sluzby.
     * 
     * @param responseData Vysledek, ktery vratila webova sluzba volanim metody {@link #callService(Object)}.
     */
    protected abstract void processResponseData(RES responseData) throws DemoException;

    /**
     * Spolecna hlavicka vsech externich zadosti nebo dotazu (webovych sluzeb). <br/>
     * Vytvori zakladn strukturu zadosti pro sluzbu ISZR. {@link ZadostInfoIszrType} obsahuje informace
     * o volajici strane a danem pozadavku. Na zaklade techto dat ISZR rozhodne, o tom, jestli ma volajici strana pravo vykonat volanou sluzbu.
     * 
     * @return Vyplnene informace o volajici strane.
     */
    protected ZadostInfoIszrType createZadostInfo()
    {
        ZadostInfoIszrType result = new ZadostInfoIszrType();

        result.setCasZadosti(new GregorianCalendar());
        result.setAgenda(getConfig().getProperty("agenda"));
        result.setAgendovaRole(getConfig().getProperty("agendovaRole"));
        result.setOvm(getConfig().getProperty("ovm"));
        result.setAis(Integer.valueOf(getConfig().getProperty("ais")));
        result.setAgendaZadostId(UUID.randomUUID().toString());

        return result;
    }

//    /**
//     * Vytvori proxy na dany port service.
//     * 
//     * @param <T> Trida proxy service
//     * @param <P> Interface portu
//     * @param serviceClass proxy trida service
//     * @param serviceNo Kod ISZR webove sluzby
//     * @param wsdl Jmeno WSDL souboru
//     * @param portName Jmeno portu
//     * @return Proxy na port service
//     * @throws DemoException
//     */
//    @SuppressWarnings({ "unchecked" })
//    protected <T extends Service, P> P createServicePort(Class<T> serviceProxy, String serviceNo, String wsdl, String portName) throws DemoException
//    {
//        WebServiceClient webServiceClient = serviceProxy.getAnnotation(WebServiceClient.class);
//        final String TARGET_ENDPOINT_ADDRESS = MessageFormat.format(getConfig().getProperty("iszr.location"), serviceNo);
//        final String TARGET_SERVICE_NAMESPACE = webServiceClient.targetNamespace();
//        final String TARGET_SERVICE_NAME = webServiceClient.name();
//        final String TARGET_WSDL_LOCATION = MessageFormat.format(getConfig().getProperty("wsdl.location"), wsdl);
//
//        // get service
//        T service;
//        try
//        {
//            Constructor<T> constructor = serviceProxy.getConstructor(URL.class, QName.class);
//            service = constructor.newInstance(new URL(TARGET_WSDL_LOCATION), new QName(TARGET_SERVICE_NAMESPACE, TARGET_SERVICE_NAME));
//        }
//        catch (MalformedURLException e)
//        {
//            throw new DemoException("Wrong target address: " + TARGET_ENDPOINT_ADDRESS, e);
//        }
//        catch (Exception e)
//        {
//            throw new IllegalStateException("Can't create instance of service proxy for target: " + TARGET_ENDPOINT_ADDRESS, e);
//        }
//
//        // get port
//        P port = null;
//        for (Method item : service.getClass().getDeclaredMethods())
//        {
//            if (item.getParameterTypes().length == 0)
//            {
//                WebEndpoint webEndpoint = item.getAnnotation(WebEndpoint.class);
//                if (webEndpoint != null && portName.equals(webEndpoint.name()))
//                {
//                    try
//                    {
//                        port = (P) item.invoke(service);
//                        break;
//                    }
//                    catch (Exception e)
//                    {
//                        throw new IllegalStateException("Can't create port " + portName + " of service: " + TARGET_ENDPOINT_ADDRESS, e);
//                    }
//                }
//            }
//        }
//
//        // setup target adress and SOAP loging
//        setUpServicePort(port, serviceNo, TARGET_ENDPOINT_ADDRESS);
//
//        return port;
//    }

    /**
     * Nastavi parametry komunikace mezi klientem a webovou sluzbou.<br/>
     * Nastavi adresu webove sluzby a logovaci handler.
     * 
     * @param port Port, jehoz atributy komunikace budou nastaveny.
     * @param serviceNo Identifikator ISZR webove sluzby. Bude soucasti jmena logovacich souboru ({@link SOAPFileLoggingHandler#getServiceNo()}).
     * @param endPointAddress Adresa webove sluzby.
     */
    @SuppressWarnings("rawtypes")
    protected <T> void setUpServicePort(T port, String serviceNo, String endPointAddress)
    {
        // Ziskani binding provideru
        BindingProvider bp = ((BindingProvider) port);
        // Ziskani bindingu
        Binding binding = bp.getBinding();

        // nastaveni adresy webove sluzby
        bp.getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, endPointAddress);

        // Vlozeni handleru pro logovani SOAP zprav
        List<Handler> handlerList = binding.getHandlerChain();
        if (handlerList == null)
        {
            handlerList = new ArrayList<>();
        }
        SOAPLoggingHandler loggingHandler = new SOAPFileLoggingHandler(serviceNo, getConfig());
        handlerList.add(loggingHandler);
        binding.setHandlerChain(handlerList);
    }

}
